@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 import numpy as np
 import mvpoly.util.common
+import mvpoly.util.version
+import scipy.integrate
 
 class MVPoly(object) :
     """
@@ -68,6 +70,78 @@ class MVPoly(object) :
         else :
             return 0
 
+    def norm_L(self, p, n=None) :
+        """
+        Returns the *Lp norm* of the polynomial, the *Lp* integral over 
+        the *n*-polycircle. If the optional argument *n* is not
+        supplied, it will be taken to be the number of variables in 
+        the polynomial or 1, whichever is the larger.
+
+        This method is only implemented up to *n=3* due to the 
+        (obvious) exponential complexity of the quadrature.
+        """
+        if np.isinf(p) :
+            return self.maxmodnd()[0]
+
+        nvar = len(self.degrees)
+        if n is None :
+            n = max(nvar, 1)
+        elif nvar > n :
+            raise ValueError, \
+                "n must be at least as large as number of variables";
+
+        twopi = 2 * np.pi
+
+        def f(ts, p, poly) :
+            """
+            The integrand
+            """
+            zs = np.array([np.exp(1j * t) for t in ts])
+            pz = poly.eval(*zs)
+            return pow(abs(pz), p)
+
+        def limits(k) :
+            return [0, twopi] + [lambda *_ : 0, lambda *_ : twopi] * (k-1)
+
+        if nvar == 0 :
+            Ip = pow(abs(self[0]), p) * (twopi**n)
+        else :
+            if  n == 1 :
+                Ip = scipy.integrate.quad(lambda t : f([t], p, self), 
+                                          *limits(1))[0]
+            elif n == 2 :
+                Ip = scipy.integrate.dblquad(lambda *ts : f(ts, p, self), 
+                                             *limits(2))[0]
+            elif n == 3 :
+                Ip = scipy.integrate.tplquad(lambda *ts : f(ts, p, self), 
+                                             *limits(3))[0]
+            elif n > 3 :
+                raise NotImplementedError;
+                if not mvpoly.util.version.at_least('scipy', (0, 13, 0)) :
+                    raise NotImplementedError, "need scipy version 0.13.0";
+                Ip = scipy.integrate.nquad(lambda *ts : f(ts, p, self), 
+                                           *limits(n+1), args=None, opts=None)
+            else :
+                raise ValueError, "bad dimension %i" % (n)
+
+        return pow(Ip/(twopi**n), 1/float(p))
+
+    @property
+    def height(self) :
+        """
+        Returns the *height* of the polynomial: the largest
+        of the absolute values of the coefficients.
+        """
+        return self.norm(np.inf)
+
+    @property
+    def length(self) :
+        """
+        Returns the *length* of the polynomial: the sum of
+        absolute values of the coefficients.
+        """
+        return self.norm(1)
+
     def maxmodnd(self, epsilon = 1e-12, i0 = None) :
         """
         The maximum modulus of a complex *n*-variate polynomial on 
