@@ -5,6 +5,7 @@
 import mvpoly
 import mvpoly.cube
 import mvpoly.dict
+import mvpoly.util.version
 import numpy as np
 import unittest
 import warnings
@@ -675,6 +676,8 @@ class TestMVPolyNorm(unittest.TestCase) :
     def test_norm_zero(self) :
         for C in classes :
             p = C.zero(dtype=int)
+            self.assertTrue(p.height == 0, "height")
+            self.assertTrue(p.length == 0, "length")
             self.assertTrue(p.norm(1) == 0, "1-norm")
             self.assertTrue(p.norm(2) == 0, "2-norm")
             self.assertTrue(p.norm(np.inf) == 0, "inf-norm")
@@ -687,6 +690,61 @@ class TestMVPolyNorm(unittest.TestCase) :
             self.assertTrue(abs(p.norm(2) -  np.sqrt(2)) < 1e-10, "2-norm")
             self.assertTrue(p.norm(np.inf) == 1, "inf-norm")
             
+    def test_norm_height_simple(self) :
+        for C in classes :
+            x, y = C.monomials(2, dtype=int)
+            p = x**2 + 5*x*y - 6
+            self.assertTrue(p.height == 6, "height of simple poly")
+
+    def test_norm_length_simple(self) :
+        for C in classes :
+            x, y = C.monomials(2, dtype=int)
+            p = x**2 - 2*x*y**9 + 3
+            self.assertTrue(p.length == 6, "length of simple poly")
+
+class TestMVPolyNormLp(unittest.TestCase) :
+
+    def test_norm_Lp_zero(self) :
+        for C in classes :
+            p = C.zero(dtype=np.complex)
+            self.assertTrue(p.norm_L(1) == 0)
+            self.assertTrue(p.norm_L(2) == 0)
+
+    def test_norm_Lp_one(self) :
+        for C in classes :
+            p = C.one(dtype=np.complex)
+            self.assertAlmostEqual(p.norm_L(1), 1)
+            self.assertAlmostEqual(p.norm_L(2), 1)
+
+    def test_norm_Lp_univariate(self) :
+        for C in classes :
+            x, = C.monomials(1, dtype=np.complex)
+            p = x + 1
+            self.assertAlmostEqual(p.norm_L(1), 4/np.pi)
+            self.assertAlmostEqual(p.norm_L(2), np.sqrt(2))
+            self.assertAlmostEqual(p.norm_L(np.inf), 2)
+
+    def test_norm_Lp_bivariate(self) :
+        for C in classes :
+            x, y = C.monomials(2, dtype=np.complex)
+            p = x + y + 1
+            # numerical values confirmed Wolfram Alpha
+            self.assertAlmostEqual(p.norm_L(2), 1.73205080756)
+            self.assertAlmostEqual(p.norm_L(4), 1.96798967126)
+
+    def test_norm_Lp_trivariate(self) :
+        for C in classes :
+            x, y, z = C.monomials(3, dtype=np.complex)
+            p = x + y + z + 1
+            # numerical value confirmed by Maple 13
+            self.assertAlmostEqual(p.norm_L(2), 2)
+
+    def test_norm_Lp_bad_n(self) :
+        for C in classes :
+            x, y, z = C.monomials(3, dtype=np.complex)
+            p = x + y + z + 1
+            self.assertRaises(ValueError, p.norm_L, 2, 2)
+
 
 class TestMVPolyRudinShapiro(unittest.TestCase) :
 
@@ -724,14 +782,20 @@ class TestMVPolyRudinShapiro(unittest.TestCase) :
                 self.assertTrue(M*(1-eps) < np.sqrt(2**(m+1)), 
                                 "rudin-shapiro maxmod")
 
-    @unittest.skip("pending implementation of norm_L(p)")
+    # Littlewood, if n = 2^k - 1, P the Rudin-Shapiro polynomial 
+    # order n, then 
+    #
+    #       ||P||_L4^4  =  (1/3)((4n+1)^2 - (-1)^k (n+1))
+    # 
+    # Computational Excursions in Analysis and Number Theory
+    # Peter Borwein
     def test_rudin_shapiro_L4_norm(self) :
-        # for norm tests : Littlewood, if n = 2^k - 1, P the Rudin-Shapiro
-        # polynomial order n, then 
-        #
-        #       ||P||_L4^4  =  (1/3)((4n+1)^2 - (-1)^k (n+1))
-        # 
-        pass
+        for C in classes :
+            for k in [2, 3, 4] :
+                expected = (4.0**k) * (4.0 - (-0.5)**k)/3.0
+                p = C.rudin_shapiro(k).astype(np.complex)
+                self.assertAlmostEqual(p.norm_L(4)**4, expected)
+
 
 class TestMVPolyLehmer(unittest.TestCase) :
 
